%% LaTeX-Beamer template for KIT design
%% by Erik Burger, Christian Hammer
%% title picture by Klaus Krogmann
%%
%% version 2.1
%%
%% mostly compatible to KIT corporate design v2.0
%% http://intranet.kit.edu/gestaltungsrichtlinien.php
%%
%% Problems, bugs and comments to
%% burger@kit.edu

\documentclass[18pt]{beamer}

%include polycode.fmt
%include lhs2TeX.fmt
%include lhs2TeX.sty

%format alpha = "\alpha"
%format beta  = "\beta"
%format gamma = "\gamma"
%format delta = "\delta"
%format forall a = "\forall " a
%format <.> = "."
%format :+: = "\oplus"
%format x1
%format x2
%format a1
%format a2
%format t1
%format t2
%format s1
%format s2

%% SLIDE FORMAT

% use 'beamerthemekit' for standard 4:3 ratio
% for widescreen slides (16:9), use 'beamerthemekitwide'

\usepackage{templates/beamerthemekit}
% \usepackage{templates/beamerthemekitwide}
\setbeamertemplate{navigation symbols}{}
%\setbeamertemplate{footline}[frame number]

% change the following line to "ngerman" for German style date and logos
\selectlanguage{ngerman}
%\usepackage{ngerman}

%% TITLE PICTURE

% if a custom picture is to be used on the title page, copy it into the 'logos'
% directory, in the line below, replace 'mypicture' with the 
% filename (without extension) and uncomment the following line
% (picture proportions: 63 : 20 for standard, 169 : 40 for wide
% *.eps format if you use latex+dvips+ps2pdf, 
% *.jpg/*.png/*.pdf if you use pdflatex)

\titleimage{beamer-logo}

%% TITLE LOGO

% for a custom logo on the front page, copy your file into the 'logos'
% directory, insert the filename in the line below and uncomment it

%\titlelogo{mylogo}

% (*.eps format if you use latex+dvips+ps2pdf,
% *.jpg/*.png/*.pdf if you use pdflatex)

%% TikZ INTEGRATION

% use these packages for PCM symbols and UML classes
% \usepackage{templates/tikzkit}
% \usepackage{templates/tikzuml}

% the presentation starts here

\title{Visualisierung von Lazy Evaluation und Sharing}
\subtitle{Vortrag zur Bachelorarbeit}
\author{Dennis Felsing}

\institute{Institut f\"{u}r Programmstrukturen und Datenorganisation}

\date{2012-10-30}

% Bibliography

\usepackage[citestyle=authoryear,bibstyle=numeric,hyperref,backend=biber]{biblatex}
\addbibresource{templates/example.bib}
\bibhang1em

\usepackage{amsfonts}
\usepackage[utf8]{inputenc}
%\usepackage[T1]{fontenc} % rasterized fonts on laptop..

%\usefonttheme[onlylarge]{structurebold}
%\usepackage{helvet}

\usepackage{fancyvrb}

\usepackage{tikz}
\usetikzlibrary{shapes,snakes,calendar,matrix,backgrounds,folding,automata,arrows,positioning,circuits.logic.IEC,chains}

\usepackage{amsmath}
\usepackage{booktabs} % toprule, midrule, bottomrule
\usepackage{array} % table cell suffix, prefix
\usepackage{longtable}
\usepackage{tabularx} % easy width

%\input{definitionen.tex}

\renewcommand{\#}[1]{\texttt{#1}}

\newcounter{stronglevel}
\setcounter{stronglevel}{1}
\renewcommand{\!}[1]{%
    \csname strong\roman{stronglevel}\endcsname{{%
        \advance\value{stronglevel} by 1\relax
        #1%
    }}%
}
\newcommand{\strongi}[1]{{\color{kit-green100}#1}}
\newcommand{\strongii}[1]{\textbf{#1}}

\begin{document}

\begin{frame}
  \titlepage
\end{frame}

\section*{Motivation}
\begin{frame}{Motivation}
  \begin{itemize}
    \item Manche Konzepte in \!{Haskell} schwer zu verstehen\\
      \begin{code}
ones    =   [1,1..]
ones'   =   1 : ones'
      \end{code}
    \item \!{Visualisierung} kann helfen:\\
    \begin{minipage}{0.45\textwidth}
      \center\includegraphics[scale=.3]{images/beamer-intro-3}
    \end{minipage}
    \begin{minipage}{0.45\textwidth}
      \begin{overlayarea}{\textwidth}{0.4\textheight}
      \vspace{-3em}
      \includegraphics<1>[scale=.3]{images/beamer-intro-1-0}
      \includegraphics<2>[scale=.3]{images/beamer-intro-1-1}
      \includegraphics<3>[scale=.3]{images/beamer-intro-1-2}
      \includegraphics<4->[scale=.3]{images/beamer-intro-1-3}
      \end{overlayarea}
    \end{minipage}
    \visible<5>{
      \item Ziel: \!{Sharing} und \!{Lazy Evaluation}\\
            intuitiv sichtbar machen
    }
  \end{itemize}
  \vspace{5em}
\end{frame}

%\begin{frame}{Überblick}
%  \tableofcontents%[hideallsubsections]
%\end{frame}

\section{Grundlagen}
\begin{frame}{Überblick}
  \tableofcontents[hideothersubsections]
\end{frame}

\subsection{Haskell}
\begin{frame}{Haskell}
  \begin{itemize}
    \item Funktionale Programmiersprache
    %\item Funktionen sind Bürger erster Klasse
    %\item Funktionen höherer Ordnung
    \item \!{Reinheit}: Keine Seiteneffekte
    \item $\Rightarrow$ \!{Referenzielle Transparenz}: Gleiche Ausdrücke werten zu gleichem Wert aus, Zeitpunkt der Auswertung spielt keine Rolle
  \end{itemize}
\end{frame}

\subsection{Lazy Evaluation}
\begin{frame}{Lazy Evaluation}
  \begin{itemize}
    %\item Konsequenz der referenziellen Transparenz
    \item Auch bekannt als \!{call-by-need}
    \item Ausdrücke \!{so spät wie möglich} auswerten $\Rightarrow$ Wenn benötigt
    \item Spart automatisch unnötige Berechnungen:
    \begin{code}
squares       =  map (^2) [1..]

smallSquares  =  takeWhile (<= 20) squares
    \end{code}
    \item Modularität: Aufteilen in \!{Generator und Selektor}
    \visible<2>{
    \begin{minipage}{0.65\textwidth}
    \item Beispiel aus der Motivation:
    \begin{code}
ones    =   [1,1..]
    \end{code}
    \end{minipage}
    \begin{minipage}{0.2\textwidth}
    \vspace{-3em}
    \includegraphics[scale=.3]{images/beamer-intro-1-1}
    \end{minipage}
    }
  \end{itemize}
\end{frame}

\subsection{Sharing}
\begin{frame}{Sharing}
  \begin{itemize}
    \item Werte sind \!{unveränderlich}
    \item $\Rightarrow$ Müssen nicht kopiert werden
    \item Stattdessen vorhandene Werte \!{wiederverwenden}:
    \begin{code}
squares       =  map (^2) [1..]

smallSquares  =  takeWhile (<= 20) squares
    \end{code}
    \item Spart mehrfaches Auswerten:
    \begin{code}
doubleTotal = total + total
  where total = sum [1..100]
    \end{code}
    \visible<2>{
    \begin{minipage}{0.65\textwidth}
    \item Beispiel aus der Motivation:
    \begin{code}
ones'   =   1 : ones'
    \end{code}
    \end{minipage}
    \begin{minipage}{0.2\textwidth}
    \vspace{-3em}
    \includegraphics[scale=.3]{images/beamer-intro-3}
    \end{minipage}
    }
  \end{itemize}
\end{frame}

\subsection{GHC Heap}
\begin{frame}{GHC Heap: Layout}
  \begin{itemize}
    \item Alle dynamischen Objekte auf Heap
    \item Einheitliches Layout als \!{Closure}
  \end{itemize}
  \begin{tikzpicture}
  \begin{scope} [start chain=going right,node distance=-0.15mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex,
      inner sep=1ex,align=center
    },
  }
  \node (pointer) [on chain] {Zeiger auf Closure};
  \node (header) [node distance=8ex, on chain, simple node] {Header};
  \node (payload) [on chain, simple node,text width=20ex] {Payload};

  \end{scope}
  \begin{scope}[start chain=going below, node distance=-0.15mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=15ex,
      inner sep=1ex,align=center
    },
  }
  \node[node distance=4ex,on chain,below right=of header] {Info table};
  \node[node distance=1ex,on chain,simple node] {Layout};
  \node[on chain,simple node] {Closure Type};
  \node[on chain,simple node] {SRT Bitmap};
  \node (ec) [on chain,simple node,dashed] {Entry Code};
  \end{scope}

  \draw[-triangle 45] (pointer.east) -- (header.west);
  \draw[*-triangle 45] ($ (header.base) !.25! (header.south) $) ||- (ec.north west);

  \end{tikzpicture}
\end{frame}

\begin{frame}{GHC Heap: Closures}
  Wichtige Closure-Typen:
  \begin{itemize}
    \item \!{Data Constructor}\only<2>{\\
      \begin{code}
        data Maybe a = Nothing | Just a
      \end{code}
    }
    \item \!{Function} und Partial Application\only<3>{\\
      \begin{code}
        square x = x * x
      \end{code}
    }
    \item \!{Thunk} und General Application\only<4>{\\
      \begin{code}
        head [1,2,3]
      \end{code}
    }
    \item Byte Code Object
  \end{itemize}
\end{frame}

\section{ghc-vis: Tool zur Visualisierung}
\begin{frame}{ghc-vis}
  \tableofcontents[currentsection,hideothersubsections]
\end{frame}

\subsection{Design}
\begin{frame}{Design}
  \includegraphics[scale=.8]{images/design2.pdf}
  %\begin{itemize}
  %  \item \!{ghc-vis} läuft in Thread in GHCi
  %  \item Interaktion über Kommandozeilenbefehle
  %  \item Auswertung auch durch Mausklick
  %\end{itemize}
\end{frame}

\begin{frame}{Design: Heap-Map}
  \begin{itemize}
    \item ghc-vis benötigt Repräsentation des relevanten Teils des GHC-Heaps: \!{Heap-Map}
    \item Folge rekursiv Zeigern in Closures
    \item Stoppe bei bereits bekannten Closures
    \item Ansichten werden aus Heap-Map generiert
  \end{itemize}
\end{frame}

\subsection{Ansichten}
\begin{frame}{Ansichten}
  \begin{minipage}{0.45\textwidth}
    \begin{verbatim}
ghci> let a = "foo"
ghci> let b = a ++ a
ghci> head b
'f'
    \end{verbatim}
    \centering
    \vspace{3em}
    \includegraphics[scale=.3]{images/lists-linear.pdf}
  \end{minipage}
  \begin{minipage}{0.45\textwidth}
    \centering
    \includegraphics[scale=.35]{images/lists-graph.pdf}
  \end{minipage}
  \begin{minipage}{0.45\textwidth}
    \centering
    Lineare Ansicht
  \end{minipage}
  \begin{minipage}{0.45\textwidth}
    \centering
    Graph-Ansicht
  \end{minipage}
\end{frame}

%\subsection{Probleme und Lösungen}
%  \begin{code}
%x = [1..2]
%  \end{code}
\subsection{Implementierung}
\begin{frame}{Implementierung: Garbage Collection}
  \only<1>{
  \begin{tikzpicture}[>=stealth',shorten >=1pt]
  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node[node distance=4ex,on chain] {Heap};
  \node [node distance=1ex,on chain, simple node, draw=white] {};
  \node [node distance=1ex,on chain, simple node, draw=white] {};
  \node [node distance=1ex,on chain, simple node, draw=white] {};
  \node [node distance=1ex,on chain, simple node, draw=white] {};
  \node (bc) [node distance=1ex,on chain,simple node] {BCO};
  \node (ap) [on chain,simple node] {AP};

  \node (x) [node distance=20ex, right of=ap] {x};
  \end{scope}

  \path[->] (ap.east) edge [out=0, in=0] (bc.east);
  \path[->] (x.west) edge [out=180, in=0] ($ (ap.east) !.5! (ap.south east) $);
  \end{tikzpicture}
  }
  \only<2>{
  \begin{tikzpicture}[>=stealth',shorten >=1pt]
  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (heap) [node distance=4ex,on chain] {Heap};
  \node (i2) [node distance=1ex,on chain,simple node] {2};
  \node (c2) [node distance=1ex,on chain,simple node] {(:)};
  \node (i1) [node distance=1ex,on chain,simple node] {1};
  \node (c1) [node distance=1ex,on chain,simple node] {(:)};
  \node (bc) [node distance=1ex,on chain,simple node] {BCO};
  \node (bh) [on chain,simple node] {Blackhole};
  \end{scope}

  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (stack) [node distance=20ex,on chain, right of=heap] {Stack};
  \node (em) [node distance=1ex,on chain, simple node] {[\hspace{1mm}]};

  \node (x) [node distance=20ex, right of=ap] {x};
  \end{scope}

  \path[->] (bh.east) edge [out=0, in=0] ($ (c1.east) !.5! (c1.south east) $);
  \path[->] ($ (c1.east) !.5! (c1.north east) $) edge [out=0, in=0] (i1.east);
  \path[->] (c1.east) edge [out=0, in=0] ($ (c2.east) !.5! (c2.south east) $);
  \path[->] ($ (c2.east) !.5! (c2.north east) $) edge [out=0, in=0] (i2.east);
  \path[->] (c2.east) edge [out=0, in=270] ($ (em.south) !.5! (em.south west) $);
  \path[->] (x.west) edge [out=180, in=0] ($ (bh.east) !.5! (bh.south east) $);
  \end{tikzpicture}
  }
  \only<3>{
  \begin{tikzpicture}[>=stealth',shorten >=1pt]
  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (heap) [node distance=4ex,on chain] {Alter Heap};
  \node (i2) [node distance=1ex,on chain,simple node] {Indirection};
  \node (c2) [node distance=1ex,on chain,simple node] {Indirection};
  \node (i1) [node distance=1ex,on chain,simple node] {Indirection};
  \node (c1) [node distance=1ex,on chain,simple node] {Indirection};
  \node (bc) [node distance=1ex,on chain,simple node] {BCO};
  \node (bh) [on chain,simple node] {Blackhole};
  \end{scope}

  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (stack) [node distance=20ex,on chain, right of=heap] {Stack};
  \node (em) [node distance=1ex,on chain, simple node] {[\hspace{1mm}]};

  \node (x) [node distance=20ex, right of=ap] {x};
  \end{scope}

  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (nheap) [node distance=20ex,on chain, right of=stack] {Neuer Heap};
  \node (ni2) [node distance=1ex,on chain,simple node] {2};
  \node (nc2) [node distance=1ex,on chain,simple node] {(:)};
  \node (ni1) [node distance=1ex,on chain,simple node] {1};
  \node (nc1) [node distance=1ex,on chain,simple node] {(:)};
  \end{scope}

  \path[->] ($ (nc1.east) !.5! (nc1.north east) $) edge [out=0, in=0] (ni1.east);
  \path[->] (nc1.east) edge [out=0, in=0] ($ (nc2.east) !.5! (nc2.south east) $);
  \path[->] ($ (nc2.east) !.5! (nc2.north east) $) edge [out=0, in=0] (ni2.east);
  \path[->] ($ (nc2.west) !.5! (nc2.north west) $) edge [out=180, in=270] ($ (em.south) !.5! (em.south east) $);

  \path[->] (bh.east) edge [out=0, in=0] ($ (c1.east) !.5! (c1.south east) $);
  \path[->] (c1.east) edge [out=0, in=180] (nc1.west);
  \path[->] (i1.east) edge [out=0, in=180] (ni1.west);
  \path[->] (c2.east) edge [out=0, in=180] (nc2.west);
  \path[->] (i2.east) edge [out=0, in=180, bend left] (ni2.west);

  \path[->] (x.east) edge [out=0, in=180] ($ (nc1.west) !.5! (nc1.south west) $);
  \end{tikzpicture}
  }
  \only<4>{
  \begin{tikzpicture}[>=stealth',shorten >=1pt]
  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (heap) [node distance=4ex,on chain,text=white] {Alter Heap};
  \node (i2) [node distance=1ex,on chain,simple node,draw=white] {};
  \node (c2) [node distance=1ex,on chain,simple node,draw=white] {};
  \node (i1) [node distance=1ex,on chain,simple node,draw=white] {};
  \node (c1) [node distance=1ex,on chain,simple node,draw=white] {};
  \node (bc) [node distance=1ex,on chain,simple node,draw=white] {};
  \node (bh) [on chain,simple node,draw=white] {};
  \end{scope}

  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (stack) [node distance=20ex,on chain, right of=heap] {Stack};
  \node (em) [node distance=1ex,on chain, simple node] {[\hspace{1mm}]};

  \node (x) [node distance=20ex, right of=ap] {x};
  \end{scope}

  \begin{scope}[start chain=going below, node distance=2mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2ex,text depth=1ex, text width=10ex,
      inner sep=1ex,align=center
    },
  }
  \node (nheap) [node distance=20ex,on chain, right of=stack] {Heap};
  \node (ni2) [node distance=1ex,on chain,simple node] {2};
  \node (nc2) [node distance=1ex,on chain,simple node] {(:)};
  \node (ni1) [node distance=1ex,on chain,simple node] {1};
  \node (nc1) [node distance=1ex,on chain,simple node] {(:)};
  \end{scope}

  \path[->] ($ (nc1.east) !.5! (nc1.north east) $) edge [out=0, in=0] (ni1.east);
  \path[->] (nc1.east) edge [out=0, in=0] ($ (nc2.east) !.5! (nc2.south east) $);
  \path[->] ($ (nc2.east) !.5! (nc2.north east) $) edge [out=0, in=0] (ni2.east);
  \path[->] ($ (nc2.west) !.5! (nc2.north west) $) edge [out=180, in=270] ($ (em.south) !.5! (em.south east) $);

  \path[->] (x.east) edge [out=0, in=180] ($ (nc1.west) !.5! (nc1.south west) $);
  \end{tikzpicture}
  }
\end{frame}

\begin{frame}{Implementierung: Garbage Collection}
  \only<1>{
    Ausschnitt aus der Heap-Map vor Garbage Collection:
\begin{verbatim}
[...
,(0x00007f732e8dbc40/2, (Nothing, ConsClosure {
  ptrArgs = [0x00007f732e8dbce0,0x00007f732e8dbcc8/2],
  dataArgs = [], name = ":"}))
,(0x00007f732dac5380, (Just "x", BlackholeClosure {
  indirectee = 0x00007f732e8dbc40/2}))
,...]
\end{verbatim}}
  \only<2->{
    Ausschnitt aus \!{derselben} Heap-Map nach Garbage Collection:
\begin{verbatim}
[...
,(0x00007f732dcad4b0/2, (Nothing, ConsClosure {
  ptrArgs = [0x00007f732dcad890,0x00007f732dcad878/2],
  dataArgs = [], name = ":"}))
,(0x00007f732dcad4b0/2, (Just "x", BlackholeClosure {
  indirectee = 0x00007f732dcad4b0/2}))
,...]
\end{verbatim}}
    \visible<3>{Lösung: Garbage Collection vor Erzeugen der Ansicht erzwingen}
\end{frame}

%\begin{frame}{Implementierung}
%  \framesubtitle{Changing Pointers}
%  \begin{itemize}
%    \item a
%  \end{itemize}
%\end{frame}

\begin{frame}{Implementierung: Byte Code Objects}
  \only<1>{
  Vollständige BCO-Informationen:\\
  \begin{center}\includegraphics[width=.75\textwidth]{images/bco.pdf}\end{center}
  }
  \only<2>{
  Keine BCO-Informationen:\\
  \begin{center}\includegraphics[scale=.3]{images/bco2.pdf}\end{center}
  \vspace{15em}
  }
  \only<3>{
  $\Rightarrow$ \!{Relevante} BCO-Informationen auswählen:\\
  \begin{center}\includegraphics[scale=.3]{images/bco3.pdf}\end{center}
  \vspace{15em}
  }
\end{frame}

%\begin{frame}{Implementierung}
%  \framesubtitle{Loading and Reloading Files}
%  \begin{itemize}
%    \item a
%  \end{itemize}
%\end{frame}

\section{Evaluierung}
\begin{frame}{Evaluierung}
  \tableofcontents[currentsection,hideothersubsections]
\end{frame}

\subsection{Vergleich}
\begin{frame}{Vergleich: :print}
  \begin{itemize}
    \item \!{:print} Bestandteil von GHCi
    \item Kein Sharing\\
    $\Rightarrow$ Keine unendlichen Datenstrukturen
  \end{itemize}
  \begin{verbatim}
ghci> let a = "foo"
ghci> let b = a ++ a
ghci> head b
'f'
  \end{verbatim}
  \begin{minipage}{.53\textwidth}
    \begin{verbatim}
ghci> :print a
a = 'f' : (_t1::[Char])
ghci> :print b
b = 'f' : (_t2::[Char])
    \end{verbatim}
  \end{minipage}
  \visible<2>{
  \hspace{1em}
  \begin{minipage}{.45\textwidth}
    \includegraphics[scale=.3]{images/lists-linear.pdf}
  \end{minipage}
  }
\end{frame}

\begin{frame}{Vergleich: vacuum}
  \begin{minipage}{.6\textwidth}
    \begin{itemize}
      \item Kein Sharing hinter unausgewerteten Ausdrücken
      \item Nicht interaktiv
    \end{itemize}
    \includegraphics[width=\textwidth]{images/vacuum2.pdf}
  \end{minipage}
  \visible<2>{
  \begin{minipage}{.38\textwidth}
    \includegraphics[scale=.3]{images/lists-graph.pdf}
  \end{minipage}
  }
\end{frame}

\subsection{Demonstration}
\begin{frame}{Demonstration}
  % 1) a++b: Linear view
  % 2) Fibonacci Zahlen: Beides
  % 3) Double Linked List: Erst dll2, dann dll: Graph view
  \includegraphics[width=\textwidth]{images/demo}
\end{frame}

\section*{Abschluss}
\begin{frame}{Abschluss}
  \begin{itemize}
    \item Live Datenstrukturen beobachten
    \item Integriert in GHCi
    \item Verständnis \!{Lazy Evaluation} und \!{Sharing}
    \item Einsatz in Lehre und Entwicklung in Haskell
  \end{itemize}
  \vspace{2em}

  Zukünftige Arbeit:
  \begin{itemize}
    \item Besser nach relevanten Datenstrukturen filtern
    \item Informationen über Typen
    \item Substrukturen aus ghc-vis in GHCi als Variablen
  \end{itemize}
\end{frame}

\appendix
\beginbackup
\begin{frame}{GHC Heap}
  \framesubtitle{Garbage Collector}
  \begin{itemize}
    \item \!{Two-Space Stop-and-Copy Collector}
    \item Alle benötigten Closures aus einer Hälfte des Heaps in andere kopieren
    \item Pointer in Closures verfolgen um weitere Closures zu finden
  \end{itemize}
\end{frame}

\begin{frame}{GHC Heap: Auswertung}
  \begin{itemize}
    \item Closure wird durch \!{Weak-Head Normal Form} ersetzt\\
    %\item Äußerste Closure ausgewertet
    %\item Data Constructors, Function Closures und Partial Applications immer in WHNF
    %\item Thunks und Generic Applications können weiter ausgewertet werden
  \end{itemize}
  \begin{code}
x = [1..3]
  \end{code}
  \begin{minipage}[t]{.3\textwidth}
    \centering
    Unausgewertet
    \includegraphics[scale=.3]{images/beamer-whnf-0}
  \end{minipage}
  \begin{minipage}[t]{.3\textwidth}
    \centering
    WHNF
    \includegraphics[scale=.3]{images/beamer-whnf-1}
  \end{minipage}
  \begin{minipage}[t]{.3\textwidth}
    \centering
    Normalform
    \includegraphics[scale=.3]{images/beamer-whnf-2}
  \end{minipage}
\end{frame}

\begin{frame}{GHC Heap: Auswertung}
  \begin{minipage}[t]{.3\textwidth}
    \centering
    Unausgewertet
    \includegraphics[scale=.3]{images/beamer-whnf-0}
  \end{minipage}
  \begin{minipage}[t]{.3\textwidth}
    \centering
    WHNF
    \includegraphics[scale=.3]{images/beamer-whnf-1}
  \end{minipage}
  \begin{minipage}[t]{.3\textwidth}
    \centering
    Normalform
    \includegraphics[scale=.3]{images/beamer-whnf-2}
  \end{minipage}
  \begin{itemize}
    \item \!{Data Constructor} \only<2->{$\Rightarrow$ WHNF}
    \item \!{Function Closure} und Partial Application \only<3->{$\Rightarrow$ WHNF}
    \item \!{Thunk} und General Application \only<4->{$\Rightarrow$ Unausgewertet}
  \end{itemize}
\end{frame}

\backupend

\end{document}
